/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd.discovery;

import java.util.logging.Level;

import java.util.*;

import com.ibm.j2ca.extension.emd.*;
import com.ibm.j2ca.extension.emd.discovery.*;
import com.ibm.j2ca.extension.emd.discovery.properties.*;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.sample.twineball.emd.*;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.MetadataConfigurationType;
import commonj.connector.metadata.discovery.properties.PropertyGroup;

/**
 * This allows us to select the operations we want, as well as specify where the schemas are generated and our maximum result set limit.
 */
public class TwineBallMetadataSelection extends WBIMetadataSelectionImpl {

    public static final String RELATIVEPATH = "RelativePath";
    public static final String MAXRECORDS = "MaxRecords";

    public String METHODNAME = null;
	public String CLASSNAME = "TwineBallMetadataSelection";
	PropertyNameHelper helper;
	private LogUtils logUtils;
	EMDUtil emdUtil;

	public TwineBallMetadataSelection(PropertyNameHelper helper){
		super(helper);
		this.helper = helper;
		this.logUtils = helper.getLogUtils();
		emdUtil = new EMDUtil();
	}

    /**
     * Create our properties for import.
     */
    public PropertyGroup createSelectionProperties() {

        WBIPropertyGroupImpl propertyGroup = null;

        METHODNAME = "createSelectionProperties()";


        try {
        	if(helper != null){
        		propertyGroup = new WBIPropertyGroupImpl(Constants.SELECTION_PROPERTIES,helper);
                propertyGroup.setDisplayName(helper.getPropertyName(Constants.SELECTIONPROPERTIES));
                propertyGroup.setDescription(helper.getPropertyDescription(Constants.SELECTIONPROPERTIES));            //-CR87212 begins            MetadataConfigurationType[] supportedValues = TwineBallMetadataDiscovery.getMetadataConfiguration();            TwineBallMetadataDiscovery.getLogUtils().trace(Level.ALL, CLASSNAME, METHODNAME, "Getting the Artifact Property");            WBISingleValuedPropertyImpl prop = EMDUtil.getBrokerProperty(supportedValues);            TwineBallMetadataDiscovery.getLogUtils().trace(Level.ALL, CLASSNAME, METHODNAME, "The property values are " + prop.getValue());            propertyGroup.addProperty(prop);            //--CR87212 ends
                //-CR87212 begins
                MetadataConfigurationType[] supportedValues = helper.getMetadataConfiguration();
                this.logUtils.trace(Level.ALL, CLASSNAME, METHODNAME, "Getting the Artifact Property");
                WBISingleValuedPropertyImpl prop = emdUtil.getBrokerProperty(supportedValues,helper);
                this.logUtils.trace(Level.ALL, CLASSNAME, METHODNAME, "The property values are " + prop.getValue());
                propertyGroup.addProperty(prop);
                //--CR87212 ends
                ServiceTypeSingleProperty operationProp = createNamespaceProperty(propertyGroup);
        		createMaxRecordProperty(propertyGroup);
        		createRelativePathProperty(propertyGroup);


			MetadataConfigurationType[] types = WBIMetadataDiscoveryImpl.getMetadataConfiguration();

			int selectedCount = 0;
			ArrayList arlSelected = new ArrayList();
			String strSelected[] = null;

			if (types != null) {
				for (int i = 0; i < types.length; i++) {

					if (types[i].equals(MetadataConfigurationType.INBOUND_SERVICE)) {
						selectedCount++;
						arlSelected.add(MetadataConfigurationType.INBOUND_SERVICE.toString());
					}
					if (types[i].equals(MetadataConfigurationType.OUTBOUND_SERVICE)) {
						selectedCount++;
						arlSelected.add(MetadataConfigurationType.OUTBOUND_SERVICE.toString());
					}
				}
			}

			String[] values = null;
			if (selectedCount > 0) {
				values = new String[arlSelected.size()];
				arlSelected.toArray(values);
			}

			if (selectedCount == 0) {
				values = new String[2];
				values[0] = MetadataConfigurationType.OUTBOUND_SERVICE.toString();
				values[1] = MetadataConfigurationType.INBOUND_SERVICE.toString();
			}
			//Property to capture type of service description
			WBISingleValuedPropertyImpl typeProp = new WBISingleValuedPropertyImpl(Constants.SERVICETYPE, String.class); //$NON-NLS-1$ 
			typeProp.setValidValues(values);
			WBIPropertyTypeImpl type = (WBIPropertyTypeImpl) typeProp.getPropertyType();
			if (selectedCount == 1)
				type.setHidden(true);
			
			typeProp.setDisplayName(WBIMetadataDiscoveryImpl.getPropertyName(Constants.SERVICETYPE)); //$NON-NLS-1$ 
			typeProp.setDescription(WBIMetadataDiscoveryImpl.getPropertyDescription(Constants.SERVICETYPE)); //$NON-NLS-1$
			typeProp.setValue(values[0]); 
			propertyGroup.addProperty(typeProp);

                //Copy the applied properties to the new instance
                if (this.getAppliedSelectionProperties() != null)
                    emdUtil.copyValues(this.getAppliedSelectionProperties(), propertyGroup,helper);
                typeProp.addPropertyChangeListener(operationProp);
        	}
        } catch (MetadataException e) {
            throw new RuntimeException(e);
        }
        return propertyGroup;
    }

    /**
     * @param propertyGroup
     * @return
     * @throws MetadataException
     */
    private WBISingleValuedPropertyImpl createServiceTypeProperty(WBIPropertyGroupImpl propertyGroup) throws MetadataException {
    	WBISingleValuedPropertyImpl typeProp = null;
    	if(helper != null){
    		typeProp = new WBISingleValuedPropertyImpl(Constants.SERVICETYPE, String.class,helper);
 	        String[] values = { Constants.INBOUND, Constants.OUTBOUND };
 	        typeProp.setValidValues(values);
 	        typeProp.setDefaultValue(Constants.OUTBOUND);
 	        typeProp.setDisplayName(helper.getPropertyName(Constants.SERVICETYPE));
 	        typeProp.setDescription(helper.getPropertyDescription(Constants.SERVICETYPE));
 	        propertyGroup.addProperty(typeProp);
    	}
        return typeProp;
    }

    /**
     * @param propertyGroup
     * @return
     * @throws MetadataException
     */
    private ServiceTypeSingleProperty createNamespaceProperty(WBIPropertyGroupImpl propertyGroup) throws MetadataException {
    	 ServiceTypeSingleProperty operationProp = null;
    	if(helper != null){
    		WBISingleValuedPropertyImpl nameSpaceProp = new WBISingleValuedPropertyImpl(Constants.NAMESPACE, String.class,helper);
	        nameSpaceProp.setDefaultValue(Constants.TB_DEFAULT_NAMESPACE);
	        propertyGroup.addProperty(nameSpaceProp);
	        nameSpaceProp.setDisplayName(helper.getPropertyName(Constants.NAMESPACE));
	        nameSpaceProp.setDescription(helper.getPropertyDescription(Constants.NAMESPACE));
	        operationProp = getOperationPropertyGroup();
	        propertyGroup.addProperty(operationProp);
    	}
        return operationProp;
    }

    /**
     * @param propertyGroup
     * @throws MetadataException
     */
    private void createRelativePathProperty(WBIPropertyGroupImpl propertyGroup) throws MetadataException {
    	if(helper != null){
    		WBISingleValuedPropertyImpl location = new WBISingleValuedPropertyImpl(RELATIVEPATH, String.class,helper);
            location.setDisplayName(helper.getPropertyName(RELATIVEPATH));
            location.setDescription(helper.getPropertyDescription(RELATIVEPATH));
            propertyGroup.addProperty(location);
    	}
    }

    /**
     * @param propertyGroup
     * @throws MetadataException
     */
    private void createMaxRecordProperty(WBIPropertyGroupImpl propertyGroup) throws MetadataException {
    	if(helper != null){
    		MaxRecordSingleProperty maxCount = new MaxRecordSingleProperty(MAXRECORDS, Integer.class,helper);
	        maxCount.setDisplayName(helper.getPropertyName(MAXRECORDS));
	        maxCount.setDescription(helper.getPropertyDescription(MAXRECORDS));
	        maxCount.setDefaultValue(new Integer("100"));
	        propertyGroup.addProperty(maxCount);
    	}
    }

    /**
     * @return
     * @throws MetadataException
     */
    private ServiceTypeSingleProperty getOperationPropertyGroup() throws MetadataException {
    	String[] operations = null;
    	ServiceTypeSingleProperty operationProp = null;
    	if(helper != null){
    		operationProp = new ServiceTypeSingleProperty(Constants.OPERATIONS, String.class,helper);
            operations = TwineBallOperations.getOutboundOperations();
            operationProp.setValidValues(operations);
            operationProp.setDisplayName(helper.getPropertyName(Constants.OPERATIONS));
            operationProp.setDescription(helper.getPropertyDescription(Constants.OPERATIONS));
    	}
        for (int i = 0; i < operations.length; i++) {
            operationProp.addValue(operations[i]);
        }
        return operationProp;
    }
    public static String getNamespace() {
        PropertyGroup pg = getAppliedSelectionProperties();
        if (pg != null) {
            WBISingleValuedPropertyImpl namespaceProp = (WBISingleValuedPropertyImpl) pg.getProperty("NameSpace"); //$NON-NLS-1$
            if (namespaceProp.getValue() != null)
                return (String) namespaceProp.getValue();
        }
        return Constants.TB_DEFAULT_NAMESPACE;
    }

}