/*
 * Licensed Materials - Property of IBM
 * pidnumber
 * (C) Copyright IBM Corporation 2005. All Rights Reserved.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */
package com.ibm.j2ca.sample.twineball.outbound;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.resource.cci.Interaction;

import junit.framework.TestCase;

import com.ibm.despi.connector.StructuredRecord;
import com.ibm.j2ca.base.SDOFactory;
import com.ibm.j2ca.base.WBIInteractionSpec;
import com.ibm.j2ca.base.internal.WBIDefaultConnectionManager;
import com.ibm.j2ca.extension.dataexchange.sdo.DEFactorySDO;
import com.ibm.j2ca.sample.twineball.TwineBallStructuredRecord;
import com.ibm.j2ca.sample.twineball.mocks.SDOBuilder;
import commonj.sdo.DataObject;

//  NOTE:  To run this test, you need ALL the WPS libraries in your classpath.
//  Please adjust the project classpath to include all jar files in WPS/lib and WPS/plugins
//  Where WPS is the location where your WPS or WESB test environment is located.
public class InteractionTest extends TestCase {

	public void setUp() throws Exception{	
		Logger l = Logger.getLogger("com.ibm.j2ca.sample.twineball.outbound.log.adapterid");
		l.setLevel(Level.ALL);
		Logger t = Logger.getLogger("com.ibm.j2ca.sample.twineball.outbound.trace.adapterid");
		t.setLevel(Level.ALL);
		//load schemas if necessary
		SDOBuilder.loadSchemas();
	}
	
	public void testCreate() throws Exception{

			// Create input SDO and populate its values (Keys)
			DataObject data = createData(true, null);

			// Create metadata SDO
			DataObject metadata = createMetadata();

			//create a new StructuredRecord
			TwineBallStructuredRecord inputRecord = new TwineBallStructuredRecord();

			//create the data exchange factory for input
			DEFactorySDO inDataBinding = new DEFactorySDO();
			inDataBinding.setBoundObject(data);
			inputRecord.initializeInput(inDataBinding, metadata);

			//create the data exchange factory for output.  
			DEFactorySDO outDataBinding = new DEFactorySDO();

			//call the adapter with this record
			StructuredRecord outputRecord = createCustomer(inputRecord);
			
			//initialize the output
			outputRecord.initializeOutput(outDataBinding, new Object[]{metadata});
			
			//retrieve the data into the output record.
			outputRecord.getNext(true);
			
			DataObject outDataObject = (DataObject)outDataBinding.getBoundObject();
			
			System.out.println(outDataObject.getString("Primarykey"));
			assertNotNull(outDataObject.getString("Primarykey"));

	}
	
	public static  TwineBallStructuredRecord createCustomer(TwineBallStructuredRecord inputRecord)throws Exception {
		TwineBallStructuredRecord outputRecord = null;

		//create the MCF, set the properties on it.
		TwineBallManagedConnectionFactory factory = new TwineBallManagedConnectionFactory();
		factory.setAdapterID("adapterid");
		factory.setLogFilename("Log.txt");
		factory.setTraceFilename("Trace.txt");
		factory.setLogFileSize(99999);
		factory.setLogNumberOfFiles(3);
		factory.setTraceFileSize(99999);
		factory.setTraceNumberOfFiles(3);


		//create a CCI connection factory
		TwineBallConnectionFactory ccif = (TwineBallConnectionFactory) factory
				.createConnectionFactory(new WBIDefaultConnectionManager());
				TwineBallConnection connection = (TwineBallConnection) ccif
				.getConnection();

       //create an interaction spec
		WBIInteractionSpec ispec = new WBIInteractionSpec();
		ispec.setFunctionName(ispec.CREATE_OP);

		Interaction interaction = connection.createInteraction();

		//execute the interaction
		outputRecord = (TwineBallStructuredRecord)interaction.execute(ispec, inputRecord);

		return outputRecord;

		
	}
	

	private static DataObject createData(boolean wholeTree, String key) throws Exception {

		DataObject data = SDOFactory.createDataObject("http://www.ibm.com/xmlns/prod/wbi/j2ca/twineball/customer", "Customer");

		data.set("Firstname", "Travis");
		data.set("Primarykey", key);

		if (wholeTree) {
			DataObject address = SDOFactory.createDataObject("http://www.ibm.com/xmlns/prod/wbi/j2ca/twineball/address", "Address");
			address.set("Primarykey", "130");
			address.set("City", "Litchfield");
			List addressList = data.getList("Address");
			addressList.add(address);
		}
		return data;

	}

	public static DataObject createMetadata() throws Exception {

		DataObject metadata = SDOFactory.createDataObject("http://www.ibm.com/xmlns/prod/wbi/j2ca/twineball/customer", "Customer");
		return metadata;
	}



 
}